@@ -5,7 +5,7 @@ import abc
 import math
 from typing import Any, Optional, Union, List
 
-from .odxtypes import DataType
+from .odxtypes import DataType, odxstr_to_bool
 from .exceptions import DecodeError, EncodeError
 from .globals import xsi, logger
 from .decodestate import DecodeState
@@ -31,11 +31,15 @@ class DiagCodedType(abc.ABC):
                  base_data_type: Union[str, DataType],
                  dct_type: str,
                  base_type_encoding=None,
-                 is_highlow_byte_order: bool = True):
+                 is_highlow_byte_order_raw: Optional[bool] = None):
         self.base_data_type = DataType(base_data_type)
         self.dct_type = dct_type
         self.base_type_encoding = base_type_encoding
-        self.is_highlow_byte_order = is_highlow_byte_order
+        self.is_highlow_byte_order_raw = is_highlow_byte_order_raw
+
+    @property
+    def is_highlow_byte_order(self) -> bool:
+        return self.is_highlow_byte_order_raw in [None, True]
 
     def _extract_internal(self,
                           coded_message: bytes,
@@ -67,7 +71,12 @@ class DiagCodedType(abc.ABC):
                 f"Don't know how to handle bit_mask={bit_mask}.")
 
         # Apply byteorder
-        if not is_highlow_byte_order and base_data_type not in [DataType.A_UNICODE2STRING, DataType.A_BYTEFIELD, DataType.A_ASCIISTRING, DataType.A_UTF8STRING]:
+        if not is_highlow_byte_order and \
+           base_data_type not in [
+               DataType.A_UNICODE2STRING,
+               DataType.A_BYTEFIELD,
+               DataType.A_ASCIISTRING,
+               DataType.A_UTF8STRING]:
             extracted_bytes = extracted_bytes[::-1]
 
         format_letter = ODX_TYPE_TO_FORMAT_LETTER[base_data_type]
@@ -194,11 +203,11 @@ class LeadingLengthInfoType(DiagCodedType):
                  base_data_type,
                  bit_length,
                  base_type_encoding=None,
-                 is_highlow_byte_order=True):
+                 is_highlow_byte_order_raw=True):
         super().__init__(base_data_type,
                          dct_type="LEADING-LENGTH-INFO-TYPE",
                          base_type_encoding=base_type_encoding,
-                         is_highlow_byte_order=is_highlow_byte_order)
+                         is_highlow_byte_order_raw=is_highlow_byte_order_raw)
         self.bit_length = bit_length
         assert self.bit_length > 0, "A Leading length info type with bit length == 0 does not make sense."
         assert self.base_data_type in [DataType.A_BYTEFIELD, DataType.A_ASCIISTRING, DataType.A_UNICODE2STRING, DataType.A_UTF8STRING], (
@@ -269,11 +278,11 @@ class MinMaxLengthType(DiagCodedType):
                  termination,
                  max_length=None,
                  base_type_encoding=None,
-                 is_highlow_byte_order=True):
+                 is_highlow_byte_order_raw=None):
         super().__init__(base_data_type,
                          dct_type="MIN-MAX-LENGTH-TYPE",
                          base_type_encoding=base_type_encoding,
-                         is_highlow_byte_order=is_highlow_byte_order)
+                         is_highlow_byte_order_raw=is_highlow_byte_order_raw)
         assert max_length is None or min_length <= max_length
         self.min_length = min_length
         self.max_length = max_length
@@ -408,11 +417,11 @@ class ParamLengthInfoType(DiagCodedType):
                  base_data_type: Union[str, DataType],
                  length_key_id: OdxLinkId,
                  base_type_encoding=None,
-                 is_highlow_byte_order=True):
+                 is_highlow_byte_order_raw=True):
         super().__init__(base_data_type,
                          dct_type="PARAM-LENGTH-INFO-TYPE",
                          base_type_encoding=base_type_encoding,
-                         is_highlow_byte_order=is_highlow_byte_order)
+                         is_highlow_byte_order_raw=is_highlow_byte_order_raw)
         self.length_key_id = length_key_id
 
     def convert_internal_to_bytes(self, internal_value, encode_state: EncodeState, bit_position: int) -> bytes:
@@ -480,14 +489,14 @@ class StandardLengthType(DiagCodedType):
     def __init__(self,
                  base_data_type: Union[str, DataType],
                  bit_length: int,
-                 bit_mask=None,
-                 condensed: bool = False,
-                 base_type_encoding=None,
-                 is_highlow_byte_order=True):
+                 bit_mask = None,
+                 condensed: Optional[bool] = None,
+                 base_type_encoding = None,
+                 is_highlow_byte_order_raw = True):
         super().__init__(base_data_type,
                          dct_type="STANDARD-LENGTH-TYPE",
                          base_type_encoding=base_type_encoding,
-                         is_highlow_byte_order=is_highlow_byte_order)
+                         is_highlow_byte_order_raw=is_highlow_byte_order_raw)
         self.bit_length = bit_length
         self.bit_mask = bit_mask
         self.condensed = condensed
@@ -525,21 +534,14 @@ class StandardLengthType(DiagCodedType):
         return self.__repr__()
 
 
-def read_diag_coded_type_from_odx(et_element, doc_frags: List[OdxDocFragment]):
+def create_any_diag_coded_type_from_et(et_element, doc_frags: List[OdxDocFragment]):
     base_type_encoding = et_element.get("BASE-TYPE-ENCODING")
 
     base_data_type = et_element.get("BASE-DATA-TYPE")
     assert base_data_type in ["A_INT32", "A_UINT32", "A_FLOAT32", "A_FLOAT64",
                               "A_ASCIISTRING", "A_UTF8STRING", "A_UNICODE2STRING", "A_BYTEFIELD"]
 
-    if et_element.get("IS-HIGHLOW-BYTE-ORDER") is not None:
-        assert et_element.get("IS-HIGHLOW-BYTE-ORDER") in ["true", "false"]
-        is_highlow_byte_order = et_element.get(
-            "IS-HIGHLOW-BYTE-ORDER") == "true"
-        logger.debug(
-            f"HIGH-LOW-BYTE-ORDER set to {is_highlow_byte_order}")
-    else:
-        is_highlow_byte_order = True
+    is_highlow_byte_order_raw = odxstr_to_bool(et_element.get("IS-HIGHLOW-BYTE-ORDER"))
 
     dct_type = et_element.get(f"{xsi}type")
     bit_length = None
@@ -548,7 +550,7 @@ def read_diag_coded_type_from_odx(et_element, doc_frags: List[OdxDocFragment]):
         return LeadingLengthInfoType(base_data_type,
                                      bit_length=bit_length,
                                      base_type_encoding=base_type_encoding,
-                                     is_highlow_byte_order=is_highlow_byte_order)
+                                     is_highlow_byte_order_raw=is_highlow_byte_order_raw)
     elif dct_type == "MIN-MAX-LENGTH-TYPE":
         min_length = int(et_element.findtext("MIN-LENGTH"))
         max_length = None
@@ -561,7 +563,7 @@ def read_diag_coded_type_from_odx(et_element, doc_frags: List[OdxDocFragment]):
                                 max_length=max_length,
                                 termination=termination,
                                 base_type_encoding=base_type_encoding,
-                                is_highlow_byte_order=is_highlow_byte_order)
+                                is_highlow_byte_order_raw=is_highlow_byte_order_raw)
     elif dct_type == "PARAM-LENGTH-INFO-TYPE":
         # TODO: This is a bit hacky: we make an ID where the data
         # specifies a reference. The reason is that we need to store
@@ -576,17 +578,17 @@ def read_diag_coded_type_from_odx(et_element, doc_frags: List[OdxDocFragment]):
         return ParamLengthInfoType(base_data_type,
                                    length_key_id,
                                    base_type_encoding=base_type_encoding,
-                                   is_highlow_byte_order=is_highlow_byte_order)
+                                   is_highlow_byte_order_raw=is_highlow_byte_order_raw)
     elif dct_type == "STANDARD-LENGTH-TYPE":
         bit_length = int(et_element.findtext("BIT-LENGTH"))
         bit_mask = None
         if et_element.find("BIT-MASK"):
             bit_mask = et_element.findtext("BIT-MASK")
-        condensed = et_element.get("CONDENSED") == "true"
+        condensed = odxstr_to_bool(et_element.get("CONDENSED"))
         return StandardLengthType(base_data_type,
                                   bit_length,
                                   bit_mask=bit_mask,
                                   condensed=condensed,
                                   base_type_encoding=base_type_encoding,
-                                  is_highlow_byte_order=is_highlow_byte_order)
+                                  is_highlow_byte_order_raw=is_highlow_byte_order_raw)
     raise NotImplementedError(f"I do not know the diag-coded-type {dct_type}")
