@@ -9,19 +9,18 @@ from .utils import short_name_as_id
 from .dataobjectproperty import (
     DataObjectProperty,
     DtcDop,
-    read_data_object_property_from_odx,
 )
-from .endofpdufield import EndOfPduField, read_end_of_pdu_field_from_odx
-from .envdata import read_env_data_from_odx, EnvironmentData
-from .envdatadesc import read_env_data_desc_from_odx, EnvironmentDataDescription
+from .endofpdufield import EndOfPduField
+from .envdata import EnvironmentData
+from .envdatadesc import EnvironmentDataDescription
 from .globals import logger
-from .multiplexer import Multiplexer, read_mux_from_odx
+from .multiplexer import Multiplexer
 from .nameditemlist import NamedItemList
-from .structures import BasicStructure, read_structure_from_odx
-from .table import read_table_from_odx, Table
-from .units import read_unit_spec_from_odx, UnitSpec
+from .structures import BasicStructure, create_any_structure_from_et
+from .table import Table
+from .units import UnitSpec
 from .odxlink import OdxLinkId, OdxLinkDatabase, OdxDocFragment
-from .specialdata import SpecialDataGroup, read_sdgs_from_odx
+from .specialdata import SpecialDataGroup, create_sdgs_from_et
 
 if TYPE_CHECKING:
     from .diaglayer import DiagLayer
@@ -96,6 +95,90 @@ class DiagDataDictionarySpec:
         if not isinstance(self.muxs, NamedItemList):
             self.muxs = _construct_named_item_list(self.muxs)
 
+    @staticmethod
+    def from_et(et_element, doc_frags: List[OdxDocFragment]) \
+            -> "DiagDataDictionarySpec":
+        # Parse DOP-BASEs
+        data_object_props = [
+            DataObjectProperty.from_et(dop_element, doc_frags)
+            for dop_element in et_element.iterfind("DATA-OBJECT-PROPS/DATA-OBJECT-PROP")
+        ]
+
+        structures = []
+        for structure_element in et_element.iterfind("STRUCTURES/STRUCTURE"):
+            structure = create_any_structure_from_et(structure_element, doc_frags)
+            assert structure is not None
+            structures.append(structure)
+
+        end_of_pdu_fields = [
+            EndOfPduField.from_et(eofp_element, doc_frags)
+            for eofp_element in et_element.iterfind("END-OF-PDU-FIELDS/END-OF-PDU-FIELD")
+        ]
+
+        dtc_dops = []
+        for dop_element in et_element.iterfind("DTC-DOPS/DTC-DOP"):
+            dop = DataObjectProperty.from_et(dop_element, doc_frags)
+            assert isinstance(dop, DtcDop)
+            dtc_dops.append(dop)
+
+        tables = [
+            Table.from_et(table_element, doc_frags)
+            for table_element in et_element.iterfind("TABLES/TABLE")
+        ]
+
+        env_data_descs = [
+            EnvironmentDataDescription.from_et(env_data_desc_element, doc_frags)
+            for env_data_desc_element in et_element.iterfind("ENV-DATA-DESCS/ENV-DATA-DESC")
+        ]
+
+        env_data_elements = chain(
+            et_element.iterfind("ENV-DATAS/ENV-DATA"),
+            # ODX 2.0.0 says ENV-DATA-DESC could contain a list of ENV-DATAS
+            et_element.iterfind("ENV-DATA-DESCS/ENV-DATA-DESC/ENV-DATAS/ENV-DATA"),
+        )
+        env_datas = [
+            EnvironmentData.from_et(env_data_element, doc_frags)
+            for env_data_element in env_data_elements
+        ]
+
+        muxs = [
+            Multiplexer.from_et(mux_element, doc_frags)
+            for mux_element in et_element.iterfind("MUXS/MUX")
+        ]
+
+        if et_element.find("UNIT-SPEC") is not None:
+            unit_spec = UnitSpec.from_et(et_element.find("UNIT-SPEC"), doc_frags)
+        else:
+            unit_spec = None
+
+        # TODO: Parse different specs.. Which of them are needed?
+        for (path, name) in [
+            ("STATIC-FIELDS", "static fields"),
+            ("DYNAMIC-LENGTH-FIELDS/DYNAMIC-LENGTH-FIELD", "dynamic length fields"),
+            (
+                "DYNAMIC-ENDMARKER-FIELDS/DYNAMIC-ENDMARKER-FIELD",
+                "dynamic endmarker fields",
+            ),
+        ]:
+            num = len(list(et_element.iterfind(path)))
+            if num > 0:
+                logger.info(f"Not implemented: Did not parse {num} {name}.")
+
+        sdgs = create_sdgs_from_et(et_element.find("SDGS"), doc_frags)
+
+        return DiagDataDictionarySpec(
+            data_object_props=NamedItemList(short_name_as_id, data_object_props),
+            structures=NamedItemList(short_name_as_id, structures),
+            end_of_pdu_fields=NamedItemList(short_name_as_id, end_of_pdu_fields),
+            dtc_dops=NamedItemList(short_name_as_id, dtc_dops),
+            unit_spec=unit_spec,
+            tables=NamedItemList(short_name_as_id, tables),
+            env_data_descs=NamedItemList(short_name_as_id, env_data_descs),
+            env_datas=NamedItemList(short_name_as_id, env_datas),
+            muxs=NamedItemList(short_name_as_id, muxs),
+            sdgs=sdgs,
+        )
+
     def _build_odxlinks(self) -> Dict[OdxLinkId, Any]:
         odxlinks = {}
         for obj in chain(self.data_object_props,
@@ -151,86 +234,3 @@ class DiagDataDictionarySpec:
     @property
     def all_data_object_properties(self):
         return self._all_data_object_properties
-
-
-def read_diag_data_dictionary_spec_from_odx(et_element, doc_frags: List[OdxDocFragment]):
-    # Parse DOP-BASEs
-    data_object_props = [
-        read_data_object_property_from_odx(dop_element, doc_frags)
-        for dop_element in et_element.iterfind("DATA-OBJECT-PROPS/DATA-OBJECT-PROP")
-    ]
-
-    structures = []
-    for structure_element in et_element.iterfind("STRUCTURES/STRUCTURE"):
-        structure = read_structure_from_odx(structure_element, doc_frags)
-        assert structure is not None
-        structures.append(structure)
-
-    end_of_pdu_fields = [
-        read_end_of_pdu_field_from_odx(eofp_element, doc_frags)
-        for eofp_element in et_element.iterfind("END-OF-PDU-FIELDS/END-OF-PDU-FIELD")
-    ]
-
-    dtc_dops = []
-    for dop_element in et_element.iterfind("DTC-DOPS/DTC-DOP"):
-        dop = read_data_object_property_from_odx(dop_element, doc_frags)
-        assert isinstance(dop, DtcDop)
-        dtc_dops.append(dop)
-
-    tables = [
-        read_table_from_odx(table_element, doc_frags)
-        for table_element in et_element.iterfind("TABLES/TABLE")
-    ]
-
-    env_data_descs = [
-        read_env_data_desc_from_odx(env_data_desc_element, doc_frags)
-        for env_data_desc_element in et_element.iterfind("ENV-DATA-DESCS/ENV-DATA-DESC")
-    ]
-
-    env_data_elements = chain(
-        et_element.iterfind("ENV-DATAS/ENV-DATA"),
-        # ODX 2.0.0 says ENV-DATA-DESC could contain a list of ENV-DATAS
-        et_element.iterfind("ENV-DATA-DESCS/ENV-DATA-DESC/ENV-DATAS/ENV-DATA"),
-    )
-    env_datas = [
-        read_env_data_from_odx(env_data_element, doc_frags)
-        for env_data_element in env_data_elements
-    ]
-
-    muxs = [
-        read_mux_from_odx(mux_element, doc_frags)
-        for mux_element in et_element.iterfind("MUXS/MUX")
-    ]
-
-    if et_element.find("UNIT-SPEC") is not None:
-        unit_spec = read_unit_spec_from_odx(et_element.find("UNIT-SPEC"), doc_frags)
-    else:
-        unit_spec = None
-
-    # TODO: Parse different specs.. Which of them are needed?
-    for (path, name) in [
-        ("STATIC-FIELDS", "static fields"),
-        ("DYNAMIC-LENGTH-FIELDS/DYNAMIC-LENGTH-FIELD", "dynamic length fields"),
-        (
-            "DYNAMIC-ENDMARKER-FIELDS/DYNAMIC-ENDMARKER-FIELD",
-            "dynamic endmarker fields",
-        ),
-    ]:
-        num = len(list(et_element.iterfind(path)))
-        if num > 0:
-            logger.info(f"Not implemented: Did not parse {num} {name}.")
-
-    sdgs = read_sdgs_from_odx(et_element.find("SDGS"), doc_frags)
-
-    return DiagDataDictionarySpec(
-        data_object_props=NamedItemList(short_name_as_id, data_object_props),
-        structures=NamedItemList(short_name_as_id, structures),
-        end_of_pdu_fields=NamedItemList(short_name_as_id, end_of_pdu_fields),
-        dtc_dops=NamedItemList(short_name_as_id, dtc_dops),
-        unit_spec=unit_spec,
-        tables=NamedItemList(short_name_as_id, tables),
-        env_data_descs=NamedItemList(short_name_as_id, env_data_descs),
-        env_datas=NamedItemList(short_name_as_id, env_datas),
-        muxs=NamedItemList(short_name_as_id, muxs),
-        sdgs=sdgs,
-    )
